@@ -86,6 +86,7 @@ go_pixbuf_save (GOImage *image, GsfXMLOut *output)
 	gsf_xml_out_add_int (output, "rowstride", pixbuf->rowstride);
 	if (!image->data) {
 		image->data = g_new0 (guint8, image->height * pixbuf->rowstride);
+		g_return_if_fail (image->data !=NULL);
 		pixbuf_to_cairo (pixbuf);
 	}
 	gsf_xml_out_add_base64
@@ -105,9 +106,16 @@ go_pixbuf_load_attr (GOImage *image, xmlChar const *attr_name, xmlChar const *at
 static void
 go_pixbuf_load_data (GOImage *image, GsfXMLIn *xin)
 {
-	int length;
+	size_t length, expected;
 	length = gsf_base64_decode_simple (xin->content->str, strlen(xin->content->str));
-	image->data = g_memdup (xin->content->str, length);
+	expected = image->height * go_pixbuf_get_rowstride (GO_PIXBUF (image));
+	if (expected != length)
+		g_critical ("Invalid image size, expected %lu bytes, got %lu", expected, length);
+	image->data = g_malloc (expected);
+	g_return_if_fail (image->data !=NULL);
+	memcpy (image->data, xin->content->str, (length < expected)? length: expected);
+	if (length < expected) /* fill with 0 */
+		memset (image->data + length, 0, expected - length);
 }
 
 static void
@@ -119,6 +127,7 @@ go_pixbuf_draw (GOImage *image, cairo_t *cr)
 		if (image->data == NULL) {
 			/* image built from a pixbuf */
 			image->data = g_new0 (guint8, image->height * pixbuf->rowstride);
+			g_return_if_fail (image->data !=NULL);
 			pixbuf_to_cairo (pixbuf);
 		}
 		pixbuf->surface = cairo_image_surface_create_for_data (image->data,
